@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -1056,18 +1056,20 @@ arithmetic_unique(struct arithmeticparams *p, char *token, int operator)
 
 
 void
-arithmetic_add_dimension(struct arithmeticparams *p, char *token, int operator)
+arithmetic_add_dimension(struct arithmeticparams *p, char *token,
+                         int operator)
 {
-  gal_data_t *out=NULL;
+  size_t one=1;
+  gal_data_t *ref=NULL, *out=NULL;
+  size_t i, j, num, dsize[3], nbytes=0;
   gal_data_t *tmp = operands_pop(p, token);
-  size_t i, num, dsize[3], ndim=3, nbytes=0;
 
   /* Make sure the first operand is a number. */
   if(tmp->size!=1)
     error(EXIT_FAILURE, 0, "first popped operand to '%s' must be a "
           "number (specifying how many datasets to use)", token);
 
-  /* Put the value into 'num'. */
+  /* Put the first-popped value into 'num'. */
   tmp=gal_data_copy_to_new_type_free(tmp, GAL_TYPE_SIZE_T);
   num=*(size_t *)(tmp->array);
   gal_data_free(tmp);
@@ -1081,43 +1083,88 @@ arithmetic_add_dimension(struct arithmeticparams *p, char *token, int operator)
       /* Things that differ from the first dataset and the rest. */
       if(out) /* Not the first. */
         {
-          /* Basic sanity checks. */
+          /* All entries have to have the same data type. */
           if(tmp->type!=out->type)
-            error(EXIT_FAILURE, 0, "the operands to '%s' have to have the "
-                  "same data type (the inputs contain atleast two types: "
-                  "'%s' and '%s')", token, gal_type_name(tmp->type, 1),
-                  gal_type_name(out->type, 1));
-          if( tmp->ndim!=out->ndim-1
-              || tmp->dsize[0]!=out->dsize[1]
-              || tmp->dsize[1]!=out->dsize[2] )
-            error(EXIT_FAILURE, 0, "the operands to '%s' have to have the "
-                  "same size", token);
+            error(EXIT_FAILURE, 0, "the operands to '%s' must have "
+                  "the same data type (the first popped operand has a "
+                  "'%s' type, but the popped operand number %zu has "
+                  "type '%s')", token, gal_type_name(out->type, 1), i+1,
+                  gal_type_name(tmp->type, 1));
+
+          /* All entries should also have the same dimension. */
+          if( gal_dimension_is_different(ref, tmp) )
+            error(EXIT_FAILURE, 0, "the operands to '%s' must have the "
+                  "same size in all dimensions", token);
         }
       else  /* First popped operand. */
         {
           /* First popped operand, do necessary basic checks here. */
-          if(tmp->ndim!=2)
-            error(EXIT_FAILURE, 0, "currently only 2-dimensional datasets "
-                  "are acceptable for '%s', please get in touch with us at "
-                  "%s so we add functionality for different dimensions",
-                  token, PACKAGE_BUGREPORT);
+          switch(operator)
+            {
+            case ARITHMETIC_OP_ADD_DIMENSION_SLOW:
+              dsize[0]=num;
+              dsize[1]=tmp->dsize[0];
+              dsize[2]=tmp->dsize[1];
+              nbytes=gal_type_sizeof(tmp->type)*tmp->size;
+              if(tmp->ndim!=2)
+                error(EXIT_FAILURE, 0, "currently only 2-dimensional "
+                      "datasets are acceptable for '%s', please get in "
+                      "touch with us at %s so we add functionality for "
+                      "different dimensions", token, PACKAGE_BUGREPORT);
+              break;
+            case ARITHMETIC_OP_ADD_DIMENSION_FAST:
+              dsize[1]=num;
+              dsize[0]=tmp->dsize[0];
+              nbytes=gal_type_sizeof(tmp->type);
+              if(tmp->ndim!=1)
+                error(EXIT_FAILURE, 0, "currently only 1-dimensional "
+                      "datasets are acceptable for '%s', please get in "
+                      "touch with us at %s so we add functionality for "
+                      "different dimensions", token, PACKAGE_BUGREPORT);
+              break;
+            default:
+              error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
+                    "%s to address the problem. The operator code '%d' "
+                    "isn't recognized", __func__, PACKAGE_BUGREPORT,
+                    operator);
+            }
+
+          /* Allocate the size-reference dataset to simplify checking of
+             the sizes for each new dataset. This is done because this
+             dataset that is actually being popped will be freed right
+             after its values have been written into the output. We don't
+             care about any data from this dataset, and to avoid wasting
+             memory, we will simply allocate a one-element dataset. But
+             we'll many set its sizes to the size of the popped dataset. */
+          ref=gal_data_alloc(NULL, tmp->type, 1, &one, NULL, 0, -1, 1,
+                             NULL, NULL, NULL);
+          free(ref->dsize);
+          ref->ndim=tmp->ndim;
+          ref->dsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, ref->ndim, 0,
+                                          __func__, "ref->dsize");
+          for(j=0;j<ref->ndim;++j) ref->dsize[j]=tmp->dsize[j];
 
           /* Allocate the output dataset. */
-          dsize[0]=num;
-          dsize[1]=tmp->dsize[0];
-          dsize[2]=tmp->dsize[1];
-          out = gal_data_alloc(NULL, tmp->type, ndim, dsize, NULL, 0,
-                               p->cp.minmapsize, p->cp.quietmmap, NULL,
+          out = gal_data_alloc(NULL, tmp->type, tmp->ndim+1, dsize, NULL,
+                               0, p->cp.minmapsize, p->cp.quietmmap, NULL,
                                NULL, NULL);
-
-          /* Get the number of bytes in each dataset. */
-          nbytes=gal_type_sizeof(tmp->type)*tmp->size;
         }
 
       /* Copy the dataset into the higher-dimensional output. */
-      memcpy(gal_pointer_increment(out->array, (num-1-i)*tmp->size,
-                                   tmp->type),
-             tmp->array, nbytes);
+      switch(operator)
+        {
+        case ARITHMETIC_OP_ADD_DIMENSION_SLOW:
+          memcpy(gal_pointer_increment(out->array, (num-1-i)*tmp->size,
+                                       tmp->type),
+                 tmp->array, nbytes);
+          break;
+        case ARITHMETIC_OP_ADD_DIMENSION_FAST:
+          for(j=0;j<tmp->size;++j)
+            memcpy(gal_pointer_increment(out->array, j*num+num-i-1, out->type),
+                   gal_pointer_increment(tmp->array, j, out->type), nbytes);
+
+          break;
+        }
 
       /* Clean up. */
       gal_data_free(tmp);
@@ -1125,12 +1172,47 @@ arithmetic_add_dimension(struct arithmeticparams *p, char *token, int operator)
 
   /* Put the higher-dimensional output on the operands stack. */
   operands_add(p, NULL, out);
+
+  /* Clean up. */
+  gal_data_free(ref);
 }
 
 
 
 
 
+void
+arithmetic_repeat(struct arithmeticparams *p, char *token, int operator)
+{
+  size_t i, num;
+  gal_data_t *ttmp, *tmp = operands_pop(p, token);
+
+  /* Make sure the first operand is a number. */
+  if(tmp->size!=1)
+    error(EXIT_FAILURE, 0, "first popped operand to '%s' must be a "
+          "number (specifying how many datasets to use)", token);
+
+  /* Put the first-popped value into 'num'. */
+  tmp=gal_data_copy_to_new_type_free(tmp, GAL_TYPE_SIZE_T);
+  num=*(size_t *)(tmp->array);
+  gal_data_free(tmp);
+
+  /* Pop the dataset to be copied. */
+  tmp=operands_pop(p, token);
+
+  /* Add it to the stack for the desired number of times. */
+  for(i=0; i<num; ++i)
+    {
+      ttmp=gal_data_copy(tmp);
+      operands_add(p, NULL, ttmp);
+    }
+
+  /* Clean up. */
+  gal_data_free(tmp);
+}
+
+
+
 
 
 
@@ -1199,8 +1281,12 @@ arithmetic_set_operator(char *string, size_t *num_operands)
         { op=ARITHMETIC_OP_COLLAPSE_NUMBER;       *num_operands=0; }
       else if (!strcmp(string, "unique"))
         { op=ARITHMETIC_OP_UNIQUE;                *num_operands=0; }
-      else if (!strcmp(string, "add-dimension"))
-        { op=ARITHMETIC_OP_ADD_DIMENSION;         *num_operands=0; }
+      else if (!strcmp(string, "add-dimension-slow"))
+        { op=ARITHMETIC_OP_ADD_DIMENSION_SLOW;    *num_operands=0; }
+      else if (!strcmp(string, "add-dimension-fast"))
+        { op=ARITHMETIC_OP_ADD_DIMENSION_FAST;    *num_operands=0; }
+      else if (!strcmp(string, "repeat"))
+        { op=ARITHMETIC_OP_REPEAT;                *num_operands=0; }
       else
         error(EXIT_FAILURE, 0, "the argument '%s' could not be "
               "interpretted as a file name, named dataset, number, "
@@ -1334,10 +1420,15 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
           arithmetic_unique(p, operator_string, operator);
           break;
 
-        case ARITHMETIC_OP_ADD_DIMENSION:
+        case ARITHMETIC_OP_ADD_DIMENSION_SLOW:
+        case ARITHMETIC_OP_ADD_DIMENSION_FAST:
           arithmetic_add_dimension(p, operator_string, operator);
           break;
 
+        case ARITHMETIC_OP_REPEAT:
+          arithmetic_repeat(p, operator_string, operator);
+          break;
+
         default:
           error(EXIT_FAILURE, 0, "%s: a bug! please contact us at "
                 "%s to fix the problem. The code %d is not "
@@ -1483,7 +1574,7 @@ reversepolish(struct arithmeticparams *p)
   /* If the final data structure has more than one element, write it as a
      FITS file. Otherwise, print it in the standard output. */
   data=p->operands->data;
-  if(data->size==1)
+  if(data->size==1 && data->ndim==1 && p->onedasimage==0)
     {
       /* Make the string to print the number. */
       printnum=gal_type_to_string(data->array, data->type, 0);
