@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -255,9 +255,8 @@ ui_read_check_only_options(struct arithmeticparams *p)
 static void
 ui_check_options_and_arguments(struct arithmeticparams *p)
 {
-  char *filename;
-  int output_checked=0;
   gal_list_str_t *token, *hdu;
+  char *filename, *basename=NULL;
   size_t nummultiext=0, numhdus=0;
   struct gal_options_common_params *cp=&p->cp;
 
@@ -296,7 +295,8 @@ ui_check_options_and_arguments(struct arithmeticparams *p)
           /* This token is a file, count how many mult-extension files we
              have and use the first to set the output filename (if it has
              not been set). */
-          if( gal_array_name_recognized(token->v) )
+          if( gal_array_name_recognized(token->v)
+              || gal_fits_file_recognized(token->v) )
             {
               /* Increment the counter for FITS files (if they are
                  input). Recall that the 'tofile' operator can also have
@@ -305,17 +305,10 @@ ui_check_options_and_arguments(struct arithmeticparams *p)
               if( gal_array_name_recognized_multiext(token->v)  )
                 ++nummultiext;
 
-              /* If the output filename isn't set yet, then set it. */
-              if(output_checked==0)
-                {
-                  if(cp->output)
-                    gal_checkset_writable_remove(cp->output, cp->keep,
-                                                 cp->dontdelete);
-                  else
-                    p->cp.output=gal_checkset_automatic_output(cp, token->v,
-                                                               "_arith.fits");
-                  output_checked=1;
-                }
+              /* If no output name is given, we need to extract the output
+                 name from the inputs. */
+              if(cp->output==NULL && basename==NULL)
+                basename=token->v;
             }
 
           /* This token is a number. Check if a negative dash was present that
@@ -336,10 +329,18 @@ ui_check_options_and_arguments(struct arithmeticparams *p)
   /* In case no output name has been given (can happen with operators like
      'makenew' when the user doesn't set an output name explicity), use a
      default name. */
-  if(p->cp.output==NULL)
+  if(cp->output)
+    gal_checkset_writable_remove(cp->output, cp->keep, cp->dontdelete);
+  else
     {
-      gal_checkset_allocate_copy("arithmetic.fits", &p->cp.output);
-      gal_checkset_writable_remove(p->cp.output, cp->keep, cp->dontdelete);
+      if(basename)
+        p->cp.output=gal_checkset_automatic_output(cp, basename,
+                                                   "_arith.fits");
+      else
+        {
+          gal_checkset_allocate_copy("arithmetic.fits", &p->cp.output);
+          gal_checkset_writable_remove(p->cp.output, cp->keep, cp->dontdelete);
+        }
     }
 
   /* Count the number of HDU values (if globalhdu isn't given) and check if
@@ -509,6 +510,7 @@ freeandreport(struct arithmeticparams *p, struct timeval *t1)
 {
   /* Free the simple strings. */
   free(p->cp.output);
+  if(p->wcshdu) free(p->wcshdu);
   if(p->globalhdu) free(p->globalhdu);
 
   /* If there are any remaining HDUs in the hdus linked list, then
