@@ -5,7 +5,7 @@ Convolve is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -326,14 +326,16 @@ ui_read_column(struct convolveparams *p, int i0k1)
   int tformat;
   char *source;
   gal_data_t *out, *cinfo;
-  gal_list_str_t *column=NULL;
+  gal_list_str_t *lines, *column=NULL;
   size_t ncols, nrows, counter=0;
   char *hdu        = i0k1==0 ? p->cp.hdu   : p->khdu;
   char *name       = i0k1==0 ? "input"     : "kernel";
   char *filename   = i0k1==0 ? p->filename : p->kernelname;
   char *columnname = i0k1==0 ? p->column   : p->kernelcolumn;
-  gal_list_str_t *lines = gal_options_check_stdin(filename,
-                                                  p->cp.stdintimeout, name);
+
+  /* Read input from standard input when necessary (only if 'filename' is
+     not given). */
+  lines=gal_options_check_stdin(filename, p->cp.stdintimeout, name);
 
   /* If no column is specified, Convolve will abort and an error will be
      printed when the table has more than one column. If there is only one
@@ -362,37 +364,37 @@ ui_read_column(struct convolveparams *p, int i0k1)
                 "Please use the '--column' ('-c') or '--kernelcolumn' "
                 "options (depending on which dataset it is) to specify a "
                 "column. You can either give it the column number "
-                "(couting from 1), or a match/search in its meta-data (e.g., "
-                "column names).\n\n"
+                "(couting from 1), or a match/search in its meta-data "
+                "(e.g., column names).\n\n"
                 "For more information, please run the following command "
                 "(press the 'SPACE' key to go down and 'q' to return to the "
                 "command-line):\n\n"
                 "    $ info gnuastro \"Selecting table columns\"\n",
                 ( filename
                   ? gal_checkset_dataset_name(filename, hdu)
-                  : "Standard input" ));
+                  : "standard input (pipe from another program)" ));
         }
-
     }
   gal_list_str_add(&column, columnname, 0);
 
   /* Read the desired column(s). */
   out=gal_table_read(filename, hdu, lines, column, p->cp.searchin,
-                     p->cp.ignorecase, p->cp.minmapsize, p->cp.quietmmap,
-                     NULL);
+                     p->cp.ignorecase, p->cp.numthreads,
+                     p->cp.minmapsize, p->cp.quietmmap, NULL);
   gal_list_str_free(lines, 1);
 
-  /* Confirm if only one column was read (it is possible to match more than
-     one column). */
+  /* Confirm if only one column was read (it is possible that a regexp
+     '--searchin' value, gives a match to more than one column). */
   if(out->next!=NULL)
     {
       if(filename)
         source=gal_checkset_dataset_name(filename, hdu);
       else
         source="standard-input";
-      error(EXIT_FAILURE, 0, "%s: more than one column in input table mached "
-            "the search criteria. Please limit the match by specifying the "
-            "exact name (if its unique) or column number", source);
+      error(EXIT_FAILURE, 0, "%s: more than one column in input table "
+            "mached the search criteria. Please limit the match by "
+            "specifying the exact name (if its unique) or column "
+            "number", source);
     }
 
   /* Make sure it is a usable datatype. */
@@ -415,6 +417,10 @@ ui_read_column(struct convolveparams *p, int i0k1)
     gal_checkset_allocate_copy("standard-input",
                                ( i0k1==0 ? &p->filename : &p->kernelname ));
 
+  /* One-dimensional convolution is only supported in spatial-mode
+     convolution. */
+  p->domain=CONVOLVE_DOMAIN_SPATIAL;
+
   /* Clean up and return. */
   gal_list_str_free(column, 0);
   return out;
