@@ -5,7 +5,7 @@ CosmicCalculator is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2019, Free Software Foundation, Inc.
+Copyright (C) 2017-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -26,10 +26,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /* When any specific citation is necessary, please add its BibTeX (from ADS
    hopefully) to this variable along with a title decribing what this
    paper/book does for the progarm in a short line. In the following line
-   put a row of `-' with the same length and then put the BibTeX.
+   put a row of '-' with the same length and then put the BibTeX.
 
-   This macro will be used in `gal_options_print_citation' function of
-   `lib/options.c' (from the top Gnuastro source code directory). */
+   This macro will be used in 'gal_options_print_citation' function of
+   'lib/options.c' (from the top Gnuastro source code directory). */
 
 #define PROGRAM_BIBTEX ""
 
