@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2018-2019, Free Software Foundation, Inc.
+Copyright (C) 2018-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,6 +33,6 @@ void
 parse_clumps(struct mkcatalog_passparams *pp);
 
 void
-parse_median(struct mkcatalog_passparams *pp);
+parse_order_based(struct mkcatalog_passparams *pp);
 
 #endif
