@@ -5,7 +5,7 @@ MakeProfiles is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,7 +33,7 @@ struct argp_option program_options[] =
     {
       "background",
       UI_KEY_BACKGROUND,
-      "STR",
+      "FITS",
       0,
       "A background image to make the profiles on.",
       GAL_OPTIONS_GROUP_INPUT,
@@ -86,7 +86,7 @@ struct argp_option program_options[] =
     {
       "customtable",
       UI_KEY_CUSTOMTABLE,
-      "STR",
+      "FITS/TXT",
       0,
       "File for 'custom' profile.",
       GAL_OPTIONS_GROUP_INPUT,
@@ -209,7 +209,7 @@ struct argp_option program_options[] =
       UI_KEY_NUMRANDOM,
       "INT",
       0,
-      "No. of random points in Monte Carlo integration.",
+      "No. of random points in Monte Carlo integ.",
       UI_GROUP_PROFILES,
       &p->numrandom,
       GAL_TYPE_SIZE_T,
@@ -376,7 +376,7 @@ struct argp_option program_options[] =
       UI_KEY_CCOL,
       "STR/INT",
       0,
-      "Coordinate columns (one call for each dimension).",
+      "Coordinate columns (one call for each dim.).",
       UI_GROUP_CATALOG,
       &p->ccol,
       GAL_TYPE_STRLL,
@@ -391,7 +391,7 @@ struct argp_option program_options[] =
       0,
       "sersic (1), moffat (2), gaussian (3), point (4), "
       "flat (5), circumference (6), distance (7), "
-      "radial-table (8)",
+      "radial-table (8), azimuth (9).",
       UI_GROUP_CATALOG,
       &p->fcol,
       GAL_TYPE_STRING,
@@ -430,7 +430,7 @@ struct argp_option program_options[] =
       UI_KEY_PCOL,
       "STR/INT",
       0,
-      "Position angle (First X-Z-X Euler angle in 3D).",
+      "Position angle (3D: first X-Z-X Euler angle).",
       UI_GROUP_CATALOG,
       &p->pcol,
       GAL_TYPE_STRING,
@@ -508,7 +508,7 @@ struct argp_option program_options[] =
       UI_KEY_TCOL,
       "STR/INT",
       0,
-      "Truncation in units of --rcol, unless --tunitinp.",
+      "Truncation in units of --rcol.",
       UI_GROUP_CATALOG,
       &p->tcol,
       GAL_TYPE_STRING,
