@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -738,6 +738,7 @@ ui_make_sorted_if_necessary(struct statisticsparams *p)
       case UI_KEY_QUANTILE:
       case UI_KEY_QUANTFUNC:
       case UI_KEY_SIGCLIPSTD:
+      case UI_KEY_QUANTOFMEAN:
       case UI_KEY_SIGCLIPMEAN:
       case UI_KEY_SIGCLIPNUMBER:
       case UI_KEY_SIGCLIPMEDIAN:
@@ -825,7 +826,8 @@ ui_read_columns(struct statisticsparams *p)
   size_t size, ncols, nrows, counter=0;
   gal_list_str_t *incols, *columnlist=NULL;
   gal_list_str_t *lines=gal_options_check_stdin(p->inputname,
-                                                p->cp.stdintimeout, "input");
+                                                p->cp.stdintimeout,
+                                                "input");
 
   /* Merge possibly multiple calls to '-c' (each possibly separated by a
      coma) into one list. */
@@ -888,8 +890,8 @@ ui_read_columns(struct statisticsparams *p)
 
   /* Read the desired column(s). */
   cols=gal_table_read(p->inputname, p->cp.hdu, lines, columnlist,
-                      p->cp.searchin, p->cp.ignorecase, p->cp.minmapsize,
-                      p->cp.quietmmap, NULL);
+                      p->cp.searchin, p->cp.ignorecase, p->cp.numthreads,
+                      p->cp.minmapsize, p->cp.quietmmap, NULL);
   gal_list_str_free(lines, 1);
 
   /* If the input was from standard input, we'll set the input name to be
