@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2019-2021, Free Software Foundation, Inc.
+Copyright (C) 2019-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -129,10 +129,11 @@ arithmetic_operator_name(int operator)
     switch(operator)
       {
       case ARITHMETIC_TABLE_OP_SET: out="set"; break;
-      case ARITHMETIC_TABLE_OP_WCSTOIMG: out="wcstoimg"; break;
-      case ARITHMETIC_TABLE_OP_IMGTOWCS: out="imgtowcs"; break;
+      case ARITHMETIC_TABLE_OP_WCSTOIMG: out="wcs-to-img"; break;
+      case ARITHMETIC_TABLE_OP_IMGTOWCS: out="img-to-wcs"; break;
       case ARITHMETIC_TABLE_OP_DATETOSEC: out="date-to-sec"; break;
       case ARITHMETIC_TABLE_OP_DISTANCEFLAT: out="distance-flat"; break;
+      case ARITHMETIC_TABLE_OP_DATETOMILLISEC: out="date-to-millisec"; break;
       case ARITHMETIC_TABLE_OP_DISTANCEONSPHERE: out="distance-on-sphere"; break;
       default:
         error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
@@ -182,12 +183,14 @@ arithmetic_set_operator(struct tableparams *p, char *string,
   if( op==GAL_ARITHMETIC_OP_INVALID )
     {
       /* Simple operators. */
-      if(      !strcmp(string, "wcstoimg"))
+      if(      !strcmp(string, "wcs-to-img"))
         { op=ARITHMETIC_TABLE_OP_WCSTOIMG; *num_operands=0; }
-      else if( !strcmp(string, "imgtowcs"))
+      else if( !strcmp(string, "img-to-wcs"))
         { op=ARITHMETIC_TABLE_OP_IMGTOWCS; *num_operands=0; }
       else if( !strcmp(string, "date-to-sec"))
         { op=ARITHMETIC_TABLE_OP_DATETOSEC; *num_operands=0; }
+      else if( !strcmp(string, "date-to-millisec"))
+        { op=ARITHMETIC_TABLE_OP_DATETOMILLISEC; *num_operands=0; }
       else if( !strcmp(string, "distance-flat"))
         { op=ARITHMETIC_TABLE_OP_DISTANCEFLAT; *num_operands=0; }
       else if( !strcmp(string, "distance-on-sphere"))
@@ -619,17 +622,20 @@ arithmetic_distance(struct tableparams *p, gal_data_t **stack, int operator)
                      p->cp.minmapsize, p->cp.quietmmap, colname, NULL,
                      colcomment);
 
-  /* Measure the distances.  */
-  o=out->array;
-  a1=a->array; a2=a->next->array;
-  b1=b->array; b2=b->next->array;
-  if(a->size==1 || b->size==1) /* One of them is a single point. */
-    for(i=0;i<a->size;++i)
-      for(j=0;j<b->size;++j)
-        o[a->size>b->size?i:j] = distance_func(a1[i], a2[i], b1[j], b2[j]);
-  else                         /* Both have the same length. */
-    for(i=0;i<a->size;++i)     /* (all were originally from the same table) */
-      o[i] = distance_func(a1[i], a2[i], b1[i], b2[i]);
+  /* Measure the distances (if the dataset isn't empty: it can be!). */
+  if(out->size>0 && out->array)
+    {
+      o=out->array;
+      a1=a->array; a2=a->next->array;
+      b1=b->array; b2=b->next->array;
+      if(a->size==1 || b->size==1) /* One of them is a single point. */
+        for(i=0;i<a->size;++i)
+          for(j=0;j<b->size;++j)
+            o[a->size>b->size?i:j] = distance_func(a1[i], a2[i], b1[j], b2[j]);
+      else                     /* Both have the same length. */
+        for(i=0;i<a->size;++i) /* (all were originally from the same table) */
+          o[i] = distance_func(a1[i], a2[i], b1[i], b2[i]);
+    }
 
   /* Clean up and put the output dataset onto the stack. */
   gal_list_data_free(a);
@@ -649,62 +655,63 @@ arithmetic_datetosec(struct tableparams *p, gal_data_t **stack,
   size_t i, v;
   int64_t *iarr;
   gal_data_t *out;
+  double subsec=NAN;
   char *subsecstr=NULL;
-  double *darr, subsec=NAN;
+  char *unit=NULL, *name=NULL, *comment=NULL;
 
   /* Input dataset. */
   gal_data_t *in=arithmetic_stack_pop(stack, operator, NULL);
   char **strarr=in->array;
 
-  /* Output metadata. */
-  char *unit="sec";
-  char *name="UNIXSEC";
-  char *comment="Unix seconds (from 00:00:00 UTC, 1 January 1970)";
-
   /* Make sure the input has a 'string' type. */
   if(in->type!=GAL_TYPE_STRING)
     error(EXIT_FAILURE, 0, "the operand given to 'date-to-sec' "
           "should have a string type, but it is '%s'",
           gal_type_name(in->type, 1));
 
+  /* Output metadata. */
+  switch(operator)
+    {
+    case ARITHMETIC_TABLE_OP_DATETOSEC:
+      unit="sec";
+      name="UNIXSEC";
+      comment="Unix seconds (from 00:00:00 UTC, 1 January 1970)";
+      break;
+    case ARITHMETIC_TABLE_OP_DATETOMILLISEC:
+      unit="msec";
+      name="UNIXMILLISEC";
+      comment="Unix milli-seconds (from 00:00:00 UTC, 1 January 1970)";
+      break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s "
+            "to fix the problem. The operator code %d isn't "
+            "recognized", __func__, PACKAGE_BUGREPORT, operator);
+    }
+
   /* Allocate the output dataset. */
   out=gal_data_alloc(NULL, GAL_TYPE_INT64, 1, &in->size, NULL, 1,
                      p->cp.minmapsize, p->cp.quietmmap, name, unit,
                      comment);
 
-  /* Convert each input string (first try as an integer, assuming no
-     sub-second, or floating point precision).  */
-  iarr=out->array;
-  for(i=0; i<in->size; ++i)
+  /* Convert each input string into number of seconds. Note that it is
+     possible to have an empty dataset, in that case, we shouldn't do
+     anything.*/
+  if(out->size>0 && out->array)
     {
-      v=gal_fits_key_date_to_seconds(strarr[i], &subsecstr,
-                                     &subsec);
-      iarr[i] = v==GAL_BLANK_SIZE_T ? GAL_BLANK_INT64 : v;
-      if(subsecstr) break;
-    }
-
-  /* If a sub-second string was present, then save the output as double
-     precision floating point. */
-  if(subsecstr)
-    {
-      /* Free the initially allocated output (as an integer). */
-      free(subsecstr);
-      gal_data_free(out);
-
-      /* Allocate a double-precision output. */
-      out=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &in->size, NULL, 1,
-                         p->cp.minmapsize, p->cp.quietmmap, name, unit,
-                         comment);
-
-      /* Convert the date. */
-      darr=out->array;
+      iarr=out->array;
       for(i=0; i<in->size; ++i)
         {
-          subsecstr=NULL;
+          /* Read the number of seconds and sub-seconds and write into the
+             output. */
           v=gal_fits_key_date_to_seconds(strarr[i], &subsecstr,
-                                               &subsec);
-          darr[i] = v==GAL_BLANK_SIZE_T ? NAN : v;
-          if(subsecstr) { darr[i]+=subsec; free(subsecstr); }
+                                         &subsec);
+          iarr[i] = ( v==GAL_BLANK_SIZE_T
+                      ? GAL_BLANK_INT64
+                      : ( operator == ARITHMETIC_TABLE_OP_DATETOSEC
+                          ? v
+                          : (isnan(subsec)
+                             ? v*1000
+                             : v*1000 + (int64_t)(subsec*1000) ) ) );
         }
     }
 
@@ -820,7 +827,8 @@ arithmetic_operator_run(struct tableparams *p,
          arguments it uses depend on the operator. In other words, when the
          operator doesn't need three operands, the extra arguments will be
          ignored. */
-      gal_list_data_add(stack, gal_arithmetic(token->operator, p->cp.numthreads,
+      gal_list_data_add(stack, gal_arithmetic(token->operator,
+                                              p->cp.numthreads,
                                               flags, d1, d2, d3) );
 
       /* Reset the meta-data for the element that was just put on the
@@ -839,6 +847,7 @@ arithmetic_operator_run(struct tableparams *p,
           break;
 
         case ARITHMETIC_TABLE_OP_DATETOSEC:
+        case ARITHMETIC_TABLE_OP_DATETOMILLISEC:
           arithmetic_datetosec(p, stack, token->operator);
           break;
 
@@ -929,8 +938,10 @@ arithmetic_reverse_polish(struct tableparams *p, struct column_pack *outpack)
               "single value, but other columns have also been requested "
               "which have more elements/rows");
 
-      /* Set 'single->next' to NULL so it isn't treated as a list and
-         remove all metadata */
+      /* Set 'single->next' to NULL so it isn't treated as a list, and set
+         all flags to zero (the arithmetic operation may have changed what
+         the flags pointed to). */
+      single->flag=0;
       single->next=NULL;
       gal_list_data_add(&p->table, single);
     }
