@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -29,6 +29,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <stdlib.h>
 #include <unistd.h>
 
+#include <gsl/gsl_rng.h>
 #include <gsl/gsl_heapsort.h>
 
 #include <gnuastro/txt.h>
@@ -77,13 +78,63 @@ table_apply_permutation(gal_data_t *table, size_t *permutation,
 
 
 
+static void
+table_bring_to_top(gal_data_t *table, gal_data_t *rowids)
+{
+  char **strarr;
+  gal_data_t *col;
+  size_t i, *ids=rowids->array;
+
+  /* Make sure the rowids are sorted by increasing index. */
+  gal_statistics_sort_increasing(rowids);
+
+  /* Go over each column and move the desired rows to the top. */
+  for(col=table;col!=NULL;col=col->next)
+    {
+      /* For easy operation if the column is a string. */
+      strarr = col->type==GAL_TYPE_STRING ? col->array : NULL;
+
+      /* Move the desired rows up to the top. */
+      for(i=0;i<rowids->size;++i)
+        if( i != ids[i] )
+          {
+            /* Copy the contents. For strings, its just about freeing and
+               copying pointers. */
+            if(col->type==GAL_TYPE_STRING)
+              {
+                free(strarr[i]);
+                strarr[i]=strarr[ ids[i] ];
+                strarr[ ids[i] ]=NULL;
+              }
+            else
+              memcpy(gal_pointer_increment(col->array, i,      col->type),
+                     gal_pointer_increment(col->array, ids[i], col->type),
+                     gal_type_sizeof(col->type));
+          }
+
+      /* For string arrays, free the pointers of the remaining rows. */
+      if(col->type==GAL_TYPE_STRING)
+        for(i=rowids->size;i<col->size;++i)
+          if(strarr[i]) free(strarr[i]);
+
+      /* Correct the size (this should be after freeing of the string
+         pointers. */
+      col->size = col->dsize[0] = rowids->size;
+    }
+
+}
+
+
+
+
+
 static gal_data_t *
 table_selection_range(struct tableparams *p, gal_data_t *col)
 {
   size_t one=1;
   double *darr;
-  int numok=GAL_ARITHMETIC_NUMOK;
-  int inplace=GAL_ARITHMETIC_INPLACE;
+  int numok=GAL_ARITHMETIC_FLAG_NUMOK;
+  int inplace=GAL_ARITHMETIC_FLAG_INPLACE;
   gal_data_t *min=NULL, *max=NULL, *tmp, *ltmin, *gemax=NULL;
 
   /* First, make sure everything is OK. */
@@ -253,8 +304,8 @@ table_selection_equal_or_notequal(struct tableparams *p, gal_data_t *col,
   void *varr;
   char **strarr;
   size_t i, one=1;
-  int numok=GAL_ARITHMETIC_NUMOK;
-  int inplace=GAL_ARITHMETIC_INPLACE;
+  int numok=GAL_ARITHMETIC_FLAG_NUMOK;
+  int inplace=GAL_ARITHMETIC_FLAG_INPLACE;
   gal_data_t *eq, *out=NULL, *value=NULL;
   gal_data_t *arg = e0n1 ? p->notequal : p->equal;
 
@@ -338,18 +389,20 @@ table_selection_equal_or_notequal(struct tableparams *p, gal_data_t *col,
 
 
 static void
-table_selection(struct tableparams *p)
+table_select_by_value(struct tableparams *p)
 {
-  uint8_t *u;
+  gal_data_t *rowids;
   struct list_select *tmp;
-  gal_data_t *mask, *addmask=NULL;
-  gal_data_t *sum, *perm, *blmask;
-  size_t i, g, b, *s, *sf, ngood=0;
-  int inplace=GAL_ARITHMETIC_INPLACE;
+  uint8_t *u, *uf, *ustart;
+  size_t i, *s, ngood=0;
+  int inplace=GAL_ARITHMETIC_FLAG_INPLACE;
+  gal_data_t *mask, *blmask, *addmask=NULL;
+
+  /* It may happen that the input table is empty! In such cases, just
+     return and don't bother with this step. */
+  if(p->table->dsize==NULL) return;
 
   /* Allocate datasets for the necessary numbers and write them in. */
-  perm=gal_data_alloc(NULL, GAL_TYPE_SIZE_T, 1, p->table->dsize, NULL, 0,
-                      p->cp.minmapsize, p->cp.quietmmap, NULL, NULL, NULL);
   mask=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, p->table->dsize, NULL, 1,
                       p->cp.minmapsize, p->cp.quietmmap, NULL, NULL, NULL);
 
@@ -378,6 +431,10 @@ table_selection(struct tableparams *p)
           addmask=table_selection_equal_or_notequal(p, tmp->col, 1);
           break;
 
+        case SELECT_TYPE_NOBLANK:
+          addmask = gal_arithmetic(GAL_ARITHMETIC_OP_ISBLANK, 1, 0, tmp->col);
+          break;
+
         default:
           error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s "
                 "to fix the problem. The code %d is not a recognized "
@@ -385,8 +442,8 @@ table_selection(struct tableparams *p)
                 tmp->type);
         }
 
-      /* Remove any blank elements. */
-      if(gal_blank_present(tmp->col, 1))
+      /* Remove any blank elements (incase we are on a noblank column. */
+      if(tmp->type!=SELECT_TYPE_NOBLANK && gal_blank_present(tmp->col, 1))
         {
           blmask = gal_arithmetic(GAL_ARITHMETIC_OP_ISBLANK, 1, 0, tmp->col);
           addmask=gal_arithmetic(GAL_ARITHMETIC_OP_OR, 1, inplace,
@@ -416,49 +473,37 @@ table_selection(struct tableparams *p)
       gal_data_free(addmask);
     }
 
-  /* Find the final number of elements to print. */
-  sum=gal_statistics_sum(mask);
-  ngood = p->table->size - ((double *)(sum->array))[0];
+  /* Find the final number of elements to print and allocate the array to
+     keep them. */
+  uf=(u=mask->array)+mask->size;
+  ngood=0; do if(*u==0) ++ngood; while(++u<uf);
+  rowids=gal_data_alloc(NULL, GAL_TYPE_SIZE_T, 1, &ngood, NULL, 0,
+                        p->cp.minmapsize, p->cp.quietmmap, NULL,
+                        NULL, NULL);
 
-  /* Define the permutation: elements within range remain on the top of
-     the list, while the ones outside of will be placed after them
-     (starting from the index after the last good one). */
-  g=0;          /* Good indexs (starting from 0). */
-  b=ngood;      /* Bad indexs (starting from total number of good). */
-  u=mask->array;
-  sf=(s=perm->array)+perm->size;
-  do *s = *u++ ? b++ : g++; while(++s<sf);
+  /* Fill-in the finally desired row-IDs. */
+  s=rowids->array;
+  ustart=mask->array;
+  uf=(u=mask->array)+mask->size;
+  do if(*u==0) *s++ = u-ustart; while(++u<uf);
 
-  /* For a check
-  {
-    size_t i;
-    double *v=ref->array;
-    uint8_t *a=mask->array;
-    printf("ref->type: %s\n", gal_type_name(ref->type, 1));
-    for(i=0;i<ref->size;++i) printf("%u, %g\n", a[i], v[i]);
-    gal_permutation_check(perm->array, perm->size);
-  }
-  */
-
-  /* Apply the final permutation to the whole table. */
-  table_apply_permutation(p->table, perm->array, ngood, 1);
+  /* Move the desired rows to the top of the table. */
+  table_bring_to_top(p->table, rowids);
 
   /* If the sort column is not in the table (the proper range has already
      been applied to it), and we need to sort the resulting columns
      afterwards, we should also apply the permutation on the sort
      column. */
-  if(p->sortcol && p->sortin==0)
-    table_apply_permutation(p->sortcol, perm->array, ngood, 1);
+  if(p->sortcol && p->sortin==0) table_bring_to_top(p->sortcol, rowids);
 
   /* Clean up. */
   i=0;
   for(tmp=p->selectcol;tmp!=NULL;tmp=tmp->next)
     { if(p->freeselect[i]) {gal_data_free(tmp->col); tmp->col=NULL;} ++i; }
   ui_list_select_free(p->selectcol, 0);
-  gal_data_free(mask);
-  gal_data_free(perm);
   free(p->freeselect);
-  gal_data_free(sum);
+  gal_data_free(mask);
+  gal_data_free(rowids);
 }
 
 
@@ -559,47 +604,149 @@ table_sort(struct tableparams *p)
 
 
 
+/* Apply random row selection. If the returned value is 'EXIT_SUCCESS',
+   then, it was successful. Otherwise, it will return 'EXIT_FAILURE' and
+   the input won't be touched. */
+static int
+table_random_rows(gal_data_t *table, gsl_rng *rng, size_t numrandom,
+                  size_t minmapsize, int quietmmap)
+{
+  int bad;
+  gal_data_t *rowids;
+  size_t i, j, *ids, ind;
+
+  /* Sanity check. */
+  if(numrandom>table->size)
+    return EXIT_FAILURE;
+
+  /* Allocate space for the list of rows to use. */
+  rowids=gal_data_alloc(NULL, GAL_TYPE_SIZE_T, 1, &numrandom, NULL, 0,
+                        minmapsize, quietmmap, NULL, NULL, NULL);
+
+  /* Select the row numbers. */
+  ids=rowids->array;
+  for(i=0;i<numrandom;++i)
+    {
+      /* Select a random index and make sure its new. */
+      bad=1;
+      while(bad)
+        {
+          ind = gsl_rng_uniform(rng) * table->size;
+          for(j=0;j<i;++j) if(ids[j]==ind) break;
+          if(i==j) bad=0;
+        }
+      ids[i]=ind;
+    }
+
+  /* Move the desired rows to the top. */
+  table_bring_to_top(table, rowids);
+
+  /* Clean up and return. */
+  gal_data_free(rowids);
+  return EXIT_SUCCESS;
+}
+
+
+
+
+
 static void
-table_head_tail(struct tableparams *p)
+table_select_by_position(struct tableparams *p)
 {
   char **strarr;
   gal_data_t *col;
   size_t i, start, end;
+  double *darr = p->rowlimit ? p->rowlimit->array : NULL;
+
+  /* Random row selection (by position, not value). This step is
+     independent of the other operations of this function, so as soon as
+     its finished return. */
+  if(p->rowrandom)
+    {
+      if( table_random_rows(p->table, p->rng, p->rowrandom,
+                            p->cp.minmapsize, p->cp.quietmmap)
+          == EXIT_FAILURE && p->cp.quiet==0 )
+        error(EXIT_SUCCESS, 0, "'--rowrandom' not activated because "
+              "the number of rows in the table at this stage (%zu) "
+              "is smaller than the number of requested random rows "
+              "(%zu). You can suppress this message with '--quiet'",
+              p->table->size, p->rowrandom);
+      return;
+    }
+
+  /* Sanity check  */
+  if(p->rowlimit)
+    {
+      if(darr[0]>=p->table->size)
+        error(EXIT_FAILURE, 0, "the first value to '--rowlimit' (%g) "
+              "is larger than the number of rows (%zu)",
+              darr[0]+1, p->table->size);
+      else if( darr[1]>=p->table->size )
+        error(EXIT_FAILURE, 0, "the second value to '--rowlimit' (%g) "
+              "is larger than the number of rows (%zu)",
+              darr[1]+1, p->table->size);
+    }
 
   /* Go over all the columns and make the necessary corrections. */
   for(col=p->table;col!=NULL;col=col->next)
     {
-      /* If we are dealing with strings, we'll need to free the strings
-         that the columns that will not be used point to (outside the
-         allocated array directly 'gal_data_t'). We don't have to worry
-         about the space for the actual pointers (they will be free'd by
-         'free' in any case, since they are in the initially allocated
-         array).*/
+      /* FOR STRING: we'll need to free the individual strings that will
+         not be used (outside the allocated array directly
+         'gal_data_t'). We don't have to worry about the space for the
+         actual pointers (they will be free'd by 'free' in any case, since
+         they are in the initially allocated array).*/
       if(col->type==GAL_TYPE_STRING)
         {
-          /* Set the start and ending indexs. */
-          start = p->head!=GAL_BLANK_SIZE_T ? p->head        : 0;
-          end   = p->head!=GAL_BLANK_SIZE_T ? p->table->size : p->tail;
-
-          /* Free their allocated spaces. */
+          /* Parse the rows and free extra pointers. */
           strarr=col->array;
-          for(i=start; i<end; ++i) { free(strarr[i]); strarr[i]=NULL; }
+          if(p->rowlimit)
+            {
+              /* Note that the given values to '--rowlimit' started from 1,
+                 but in 'ui.c' we subtracted one from it (so at this stage,
+                 it starts from 0). */
+              start = darr[0];
+              end   = darr[1];
+              for(i=0;i<p->table->size;++i)
+                if(i<start || i>end) { free(strarr[i]); strarr[i]=NULL; }
+            }
+          else
+            {
+              /* Set the starting and ending indexs to free the allocated
+                 space of each string. */
+              start = p->head!=GAL_BLANK_SIZE_T ? p->head        : 0;
+              end   = ( p->head!=GAL_BLANK_SIZE_T
+                        ? p->table->size
+                        : p->table->size - p->tail );
+              for(i=start; i<end; ++i) { free(strarr[i]); strarr[i]=NULL; }
+            }
         }
 
-      /* For '--tail', we'll need to bring the last columns to the
-         start. Note that we are using 'memmove' because we want to be safe
-         with overlap. */
-      if(p->tail!=GAL_BLANK_SIZE_T)
-        memmove(col->array,
-                gal_pointer_increment(col->array, col->size - p->tail,
-                                      col->type),
-                p->tail*gal_type_sizeof(col->type));
+      /* Make the final adjustment. */
+      if(p->rowlimit)
+        {
+          /* Move the values up to the top and correct the size. */
+          col->size=darr[1]-darr[0]+1;
+          memmove(col->array,
+                  gal_pointer_increment(col->array, darr[0], col->type),
+                  (darr[1]-darr[0]+1)*gal_type_sizeof(col->type));
+        }
+      else
+        {
+          /* For '--tail', we'll need to bring the last columns to the
+             start. Note that we are using 'memmove' because we want to be
+             safe with overlap. */
+          if(p->tail!=GAL_BLANK_SIZE_T)
+            memmove(col->array,
+                    gal_pointer_increment(col->array, col->size - p->tail,
+                                          col->type),
+                    p->tail*gal_type_sizeof(col->type));
 
-      /* In any case (head or tail), the new number of column elements is
-         the given value. */
-      col->size = col->dsize[0] = ( p->head!=GAL_BLANK_SIZE_T
-                                    ? p->head
-                                    : p->tail );
+          /* In any case (head or tail), the new number of column elements
+             is the given value. */
+          col->size = col->dsize[0] = ( p->head!=GAL_BLANK_SIZE_T
+                                        ? p->head
+                                        : p->tail );
+        }
     }
 }
 
@@ -607,8 +754,7 @@ table_head_tail(struct tableparams *p)
 
 
 
-/*This function concatenates two table column wise .
- It  attaches catcolumn table at the back of first table */
+/* Import columns from another file/table into the working table. */
 static void
 table_catcolumn(struct tableparams *p)
 {
@@ -623,28 +769,30 @@ table_catcolumn(struct tableparams *p)
   for(filell=p->catcolumnfile; filell!=NULL; filell=filell->next)
     {
       /* Set the HDU (not necessary for non-FITS tables). */
-      if(gal_fits_name_is_fits(filell->v))
+      if(gal_fits_file_recognized(filell->v))
         {
           if(hdull) { hdu=hdull->v; hdull=hdull->next; }
           else
-            error(EXIT_FAILURE, 0, "not enough '--catcolumnhdu's (or '-u'). "
-                  "For every FITS table given to '--catcolumnfile'. A call to "
-                  "'--catcolumnhdu' is necessary to identify its "
-                  "HDU/extension");
+            error(EXIT_FAILURE, 0, "not enough '--catcolumnhdu's (or "
+                  "'-u'). For every FITS table given to "
+                  "'--catcolumnfile'. A call to '--catcolumnhdu' is "
+                  "necessary to identify its HDU/extension");
         }
       else hdu=NULL;
 
       /* Read the catcolumn table. */
-      tocat=gal_table_read(filell->v, hdu, NULL, p->catcolumns, cp->searchin,
-                           cp->ignorecase, cp->minmapsize, p->cp.quietmmap,
-                           NULL);
+      tocat=gal_table_read(filell->v, hdu, NULL, p->catcolumns,
+                           cp->searchin, cp->ignorecase,
+                           cp->numthreads, cp->minmapsize,
+                           p->cp.quietmmap, NULL);
 
       /* Check the number of rows. */
       if(tocat->dsize[0]!=p->table->dsize[0])
-        error(EXIT_FAILURE, 0, "%s: incorrect number of rows. The table given "
-              "to '--catcolumn' must have the same number of rows as the "
-              "main argument (after all row-selections have been applied), "
-              "but they have %zu and %zu rows respectively",
+        error(EXIT_FAILURE, 0, "%s: incorrect number of rows. The "
+              "table given to '--catcolumn' must have the same number "
+              "of rows as the main argument (after all row-selections "
+              "have been applied), but they have %zu and %zu rows "
+              "respectively",
               gal_fits_name_save_as_string(filell->v, hdu), tocat->dsize[0],
               p->table->dsize[0]);
 
@@ -676,6 +824,175 @@ table_catcolumn(struct tableparams *p)
 
 
 
+/* Find the HDU of the table to read. */
+static char *
+table_catrows_findhdu(char *filename, gal_list_str_t **hdull)
+{
+  char *hdu=NULL;
+
+  /* Set the HDU (not necessary for non-FITS tables). */
+  if(gal_fits_file_recognized(filename))
+    {
+      if(*hdull) { hdu=(*hdull)->v; *hdull=(*hdull)->next; }
+      else
+        error(EXIT_FAILURE, 0, "not enough '--catrowhdu's (or "
+              "'-H'). For every FITS table given to '--catrowfile'. "
+              "A call to '--catrowhdu' is necessary to identify "
+              "its HDU/extension");
+    }
+
+  /* Return the HDU. */
+  return hdu;
+}
+
+
+
+
+
+/* Preparations for adding rows: allocate final table, copy input table
+   into it, and free the input table (while checking if enough HDUs are
+   given for all the tables whose rows should be added). */
+static size_t
+table_catrows_prepare(struct tableparams *p)
+{
+  char **strarr;
+  char *hdu=NULL;
+  int tableformat;
+  gal_data_t *tmp, *out=NULL;
+  size_t i, nrows=p->table->size;
+  gal_list_str_t *filell, *hdull;
+  size_t numcols, numrows, filledrows=p->table->size;
+
+  /* Go over all the given tables and find the final number of rows. */
+  hdull=p->catrowhdu;
+  for(filell=p->catrowfile; filell!=NULL; filell=filell->next)
+    {
+      hdu=table_catrows_findhdu(filell->v, &hdull);
+      gal_table_info(filell->v, hdu, NULL, &numcols, &numrows,
+                     &tableformat);
+      nrows+=numrows;
+    }
+
+  /* Allocate the new table with the necessary number of rows, then reverse
+     the newly allocated table (its columns were added in a
+     first-in-first-out list).*/
+  for(tmp=p->table; tmp!=NULL; tmp=tmp->next)
+    {
+      /* Allocate the new column. */
+      gal_list_data_add_alloc(&out, NULL, tmp->type, 1, &nrows, NULL,
+                              0, p->cp.minmapsize, p->cp.quietmmap,
+                              tmp->name, tmp->unit, tmp->comment);
+
+      /* Put the full contents of the existing column into the new
+         column: this will be the first set of rows,  */
+      memcpy(out->array, tmp->array, tmp->size*gal_type_sizeof(tmp->type));
+
+      /* If the column type is a string, we should set the input pointers
+         to NULL to avoid freeing them later. */
+      if(tmp->type==GAL_TYPE_STRING)
+        {
+          strarr=tmp->array;
+          for(i=0;i<tmp->size;++i) strarr[i]=NULL;
+        }
+    }
+  gal_list_data_reverse(&out);
+
+  /* Clean up and return. */
+  gal_list_data_free(p->table);
+  p->table=out;
+  return filledrows;
+}
+
+
+
+
+
+/* Import rows from another set of table(s). */
+static void
+table_catrows(struct tableparams *p)
+{
+  char *hdu=NULL, **strarr;
+  gal_data_t *new, *ttmp, *tmp;
+  gal_list_str_t *filell, *hdull;
+  size_t i, colcount, ncols, ncolstest, filledrows;
+
+  /* Make sure enough HDUs are given, and allocate the final output table,
+     while filling the initiall table rows into it. */
+  filledrows=table_catrows_prepare(p);
+
+  /* Go over all the given tables and extract the same set of columns that
+     were extracted from the input table. */
+  hdull=p->catrowhdu;
+  ncols=gal_list_data_number(p->table);
+  for(filell=p->catrowfile; filell!=NULL; filell=filell->next)
+    {
+      /* Read the columns of the new table. */
+      hdu=table_catrows_findhdu(filell->v, &hdull);
+      new=gal_table_read(filell->v, hdu, NULL, p->columns,
+                         p->cp.searchin, p->cp.ignorecase,
+                         p->cp.numthreads, p->cp.minmapsize,
+                         p->cp.quietmmap, NULL);
+
+      /* Make sure that the same number of columns were extracted from this
+         table as they were from the original table. */
+      ncolstest=gal_list_data_number(new);
+      if(ncolstest!=ncols)
+        error(EXIT_FAILURE, 0, "%s: %zu column(s) were matched with "
+              "your requested columns. However, the final table "
+              "before adding rows contains %zu column(s). For "
+              "concatenating (adding) rows, the final number of "
+              "columns in all input tables should be the same. "
+              "Note that adding columns is done before adding "
+              "rows", gal_fits_name_save_as_string(filell->v, hdu),
+              ncolstest, ncols);
+
+      /* Parse all the new columns and add their contents to the already
+         allocated space of the output. */
+      colcount=1;
+      ttmp=p->table;
+      for(tmp=new; tmp!=NULL; tmp=tmp->next)
+        {
+          /* See if this column has the same type as the same column in the
+             input table. */
+          if(tmp->type!=ttmp->type)
+            error(EXIT_FAILURE, 0, "%s: column %zu has a data type of "
+                  "'%s'. However, in the final table (before adding "
+                  "rows) this column has a type of '%s'. For "
+                  "concatenating (adding) rows, the columns must have "
+                  "the same data type. Note that adding columns is "
+                  "done before adding rows. If you haven't added columns "
+                  "you can use Table's column arithmetic to change the "
+                  "data type of this column in the inputs",
+                  gal_fits_name_save_as_string(filell->v, hdu), colcount,
+                  gal_type_name(tmp->type, 1), gal_type_name(ttmp->type, 1));
+
+          /* Add the new rows and incremenet the counter. */
+          memcpy(gal_pointer_increment(ttmp->array, filledrows, ttmp->type),
+                 tmp->array, tmp->size*gal_type_sizeof(tmp->type));
+
+          /* If the column type is a string, we should set the input
+             pointers to NULL to avoid freeing them later. */
+          if(tmp->type==GAL_TYPE_STRING)
+            {
+              strarr=tmp->array;
+              for(i=0;i<tmp->size;++i) strarr[i]=NULL;
+            }
+
+          /* Take 'ttmp' to the next column and increment the counter */
+          ttmp=ttmp->next;
+          ++colcount;
+        }
+
+      /* Clean up the columns of the table and increment 'filledrows'. */
+      filledrows += new->size;
+      gal_list_data_free(new);
+    }
+}
+
+
+
+
+
 void
 table_colmetadata(struct tableparams *p)
 {
@@ -748,74 +1065,88 @@ table_colmetadata(struct tableparams *p)
 
 
 void
-table_noblank(struct tableparams *p)
+table_noblankend(struct tableparams *p)
 {
   int found;
+  gal_list_str_t *tmp;
   size_t i, j, *index;
   gal_data_t *tcol, *flag;
-  char **strarr=p->noblank->array;
   gal_list_sizet_t *column_indexs=NULL;
 
-  /* Go over the given list of given columns. */
-  for(i=0;i<p->noblank->size;++i)
+  /* Merge all possible calls to '--noblankend' into one list. */
+  gal_options_merge_list_of_csv(&p->noblankend);
+
+  /* See if all columns should be checked, or just a select few. */
+  if( gal_list_str_number(p->noblankend)==1
+      && !strcmp(p->noblankend->v,"_all") )
     {
-      /* First go through the column names and if they match, add
-         them. Note that we don't want to stop once a name is found, in
-         this scenario, if multiple columns have the same name, we should
-         use all.*/
-      j=0;
-      found=0;
-      for(tcol=p->table; tcol!=NULL; tcol=tcol->next)
-        {
-          if( tcol->name && !strcmp(tcol->name, strarr[i]) )
-            {
-              found=1;
-              gal_list_sizet_add(&column_indexs, j);
-            }
-          ++j;
-        }
-
-      /* If the given string didn't match any column name, it must be a
-         number, so parse it as a number and use that number. */
-      if(found==0)
-        {
-          /* Parse the given index. */
-          index=NULL;
-          if( gal_type_from_string((void **)(&index), strarr[i],
-                                   GAL_TYPE_SIZE_T) )
-            error(EXIT_FAILURE, 0, "column '%s' didn't match any of the "
-                  "final column names and can't be parsed as a column "
-                  "counter (starting from 1) either", strarr[i]);
-
-          /* Make sure its not zero (the user counts from 1). */
-          if(*index==0)
-            error(EXIT_FAILURE, 0, "the column number (given to the "
-                  "'--noblank' option) should start from 1, but you have "
-                  "given 0.");
-
-          /* Make sure that the index falls within the number (note that it
-             still counts from 1).  */
-          if(*index > gal_list_data_number(p->table))
-            error(EXIT_FAILURE, 0, "the final output table only has %zu "
-                  "columns, but you have given column %zu to '--noblank'. "
-                  "Recall that '--noblank' operates on the output columns "
-                  "and that you can also use output column names (if they "
-                  "have any)",
-                  gal_list_data_number(p->table), *index);
-
-          /* Everything is fine, add the index to the list of columns to
-             check. */
-          gal_list_sizet_add(&column_indexs, *index-1);
-
-          /* Clean up. */
-          free(index);
-        }
-
-      /* For a check.
-      printf("%zu\n", column_indexs->v);
-      */
+      for(i=0;i<gal_list_data_number(p->table);++i)
+        gal_list_sizet_add(&column_indexs, i);
     }
 
+  /* Only certain columns should be checked, so find/add their index. */
+  else
+    for(tmp=p->noblankend; tmp!=NULL; tmp=tmp->next)
+      {
+        /* First go through the column names and if they match, add
+           them. Note that we don't want to stop once a name is found, in
+           this scenario, if multiple columns have the same name, we should
+           use all.*/
+        j=0;
+        found=0;
+        for(tcol=p->table; tcol!=NULL; tcol=tcol->next)
+          {
+            if( tcol->name && !strcmp(tcol->name, tmp->v) )
+              {
+                found=1;
+                gal_list_sizet_add(&column_indexs, j);
+              }
+            ++j;
+          }
+
+        /* If the given string didn't match any column name, it must be a
+           number, so parse it as a number and use that number. */
+        if(found==0)
+          {
+            /* Parse the given index. */
+            index=NULL;
+            if( gal_type_from_string((void **)(&index), tmp->v,
+                                     GAL_TYPE_SIZE_T) )
+              error(EXIT_FAILURE, 0, "column '%s' didn't match any of the "
+                    "final column names and can't be parsed as a column "
+                    "counter (starting from 1) either", tmp->v);
+
+            /* Make sure its not zero (the user counts from 1). */
+            if(*index==0)
+              error(EXIT_FAILURE, 0, "the column number (given to the "
+                    "'--noblankend' option) should start from 1, but you have "
+                    "given 0.");
+
+            /* Make sure that the index falls within the number (note that
+               it still counts from 1).  */
+            if(*index > gal_list_data_number(p->table))
+              error(EXIT_FAILURE, 0, "the final output table only has "
+                    "%zu columns, but you have given column %zu to "
+                    "'--noblankend'. Recall that '--noblankend' operates "
+                    "at the end (on the output columns) and that you "
+                    "can also use output column names (if they have "
+                    "any). In case you meant a column from the input "
+                    "table, you should use '--noblank'",
+                    gal_list_data_number(p->table), *index);
+
+            /* Everything is fine, add the index to the list of columns to
+               check. */
+            gal_list_sizet_add(&column_indexs, *index-1);
+
+            /* Clean up. */
+            free(index);
+          }
+
+        /* For a check.
+           printf("%zu\n", column_indexs->v);
+        */
+      }
+
   /* Remove all blank rows from the output table, note that we don't need
      the flags of the removed columns here. So we can just free it up. */
   flag=gal_blank_remove_rows(p->table, column_indexs);
@@ -845,28 +1176,34 @@ table_noblank(struct tableparams *p)
 void
 table(struct tableparams *p)
 {
-  /* Apply a certain range (if required) to the output sample. */
-  if(p->selection) table_selection(p);
+  /* Concatenate the columns of tables (if required). */
+  if(p->catcolumnfile) table_catcolumn(p);
+
+  /* Concatenate the rows of multiple tables (if required). */
+  if(p->catrowfile) table_catrows(p);
+
+  /* Apply ranges based on row values (if required). */
+  if(p->selection) table_select_by_value(p);
 
   /* Sort it (if required). */
   if(p->sort) table_sort(p);
 
   /* If the output number of rows is limited, apply them. */
-  if(p->head!=GAL_BLANK_SIZE_T || p->tail!=GAL_BLANK_SIZE_T)
-    table_head_tail(p);
+  if( p->rowlimit
+      || p->rowrandom
+      || p->head!=GAL_BLANK_SIZE_T
+      || p->tail!=GAL_BLANK_SIZE_T )
+    table_select_by_position(p);
 
-  /* If any operations are needed, do them. */
+  /* If any arithmetic operations are needed, do them. */
   if(p->outcols)
     arithmetic_operate(p);
 
-  /* Concatenate the columns of tables (if required)*/
-  if(p->catcolumnfile) table_catcolumn(p);
-
   /* When column metadata should be updated. */
   if(p->colmetadata) table_colmetadata(p);
 
   /* When any columns with blanks should be removed. */
-  if(p->noblank) table_noblank(p);
+  if(p->noblankend) table_noblankend(p);
 
   /* Write the output. */
   gal_table_write(p->table, NULL, NULL, p->cp.tableformat, p->cp.output,
