@@ -4,7 +4,7 @@ Function to parse options and configuration file values.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2021, Free Software Foundation, Inc.
+Copyright (C) 2017-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -125,6 +125,7 @@ enum options_common_keys
   GAL_OPTIONS_KEY_INTERPONLYBLANK,
   GAL_OPTIONS_KEY_INTERPMETRIC,
   GAL_OPTIONS_KEY_INTERPNUMNGB,
+  GAL_OPTIONS_KEY_WCSLINEARMATRIX,
 };
 
 
@@ -194,9 +195,10 @@ struct gal_options_common_params
   /* Output. */
   char                 *output; /* Directory containg output.             */
   uint8_t                 type; /* Data type of output.                   */
+  uint8_t          tableformat; /* Internal code for output table format. */
+  uint8_t      wcslinearmatrix; /* WCS matrix to use (PC or CD).          */
   uint8_t           dontdelete; /* ==1: Don't delete existing file.       */
   uint8_t         keepinputdir; /* Keep input directory for auto output.  */
-  uint8_t          tableformat; /* Internal code for output table format. */
 
   /* Operating modes. */
   uint8_t                quiet; /* Only print errors.                     */
@@ -275,6 +277,10 @@ void *
 gal_options_read_searchin(struct argp_option *option, char *arg,
                           char *filename, size_t lineno, void *junk);
 
+void *
+gal_options_read_wcslinearmatrix(struct argp_option *option, char *arg,
+                                 char *filename, size_t lineno, void *junk);
+
 void *
 gal_options_read_tableformat(struct argp_option *option, char *arg,
                              char *filename, size_t lineno, void *junk);
@@ -318,6 +324,10 @@ void *
 gal_options_parse_name_and_float64s(struct argp_option *option, char *arg,
                                     char *filename, size_t lineno, void *junk);
 
+gal_data_t *
+gal_options_parse_colon_sep_csv_raw(char *instring, char *filename,
+                                    size_t lineno);
+
 void *
 gal_options_parse_colon_sep_csv(struct argp_option *option, char *arg,
                                 char *filename, size_t lineno, void *junk);
