@@ -4,7 +4,8 @@ Functions dealing with general aspects of all Gnuastro.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+     Pedram Ashofteh Ardakani <pedramardakani@pm.me>
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -34,10 +35,11 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 /* Configuration macros: */
 
-#define GAL_CONFIG_VERSION                  "0.16"
+#define GAL_CONFIG_VERSION                  "0.17"
 
 #define GAL_CONFIG_HAVE_LIBGIT2             yes
 
+#define GAL_CONFIG_HAVE_GSL_INTERP_STEFFEN  1
 #define GAL_CONFIG_HAVE_FITS_IS_REENTRANT   1
 #define GAL_CONFIG_HAVE_WCSLIB_VERSION      1
 #define GAL_CONFIG_HAVE_WCSLIB_DIS_H        1
