@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -245,12 +245,11 @@ txt_info_from_comment(char *in_line, gal_data_t **datall, char *comm_start,
 
 
       /* If 'typestr' was given, then check if this is a standard type. If
-         'typestr' wasn't specified, then the default double type code will
-         be used (see the variable definitions above). If the given type
-         isn't a standard type then ignore the line. Just note that if we
-         are dealing with the string type, we have to pull out the number
-         part first. If there is no number for a string type, then ignore
-         the line. */
+         'typestr' wasn't specified or couldn't be interpretted, then the
+         default double type code will be used (see the variable
+         definitions above). Just note that if we are dealing with the
+         string type, we have to pull out the number part first. If there
+         is no number for a string type, then ignore the line. */
       if(typestr && *typestr!='\0')
         {
           typestr=gal_txt_trim_space(typestr);
@@ -263,7 +262,8 @@ txt_info_from_comment(char *in_line, gal_data_t **datall, char *comm_start,
           else
             {
               type=gal_type_from_name(typestr);
-              if(type==GAL_TYPE_INVALID) return;
+              if(type==GAL_TYPE_INVALID)
+                type=GAL_TYPE_FLOAT64;
             }
         }
 
@@ -315,8 +315,10 @@ static size_t
 txt_info_from_first_row(char *in_line, gal_data_t **datall, int format,
                         int inplace)
 {
+  double tmpd;
+  void *tmpdptr=&tmpd;
   gal_data_t *col, *prev, *tmp;
-  size_t n=0, maxcnum=0, numtokens;
+  size_t ncol=0, maxcnum=0, numtokens;
   char *line, *token, *end, *aline=NULL;
 
   /* Make a copy of the input line if necessary. */
@@ -348,14 +350,15 @@ txt_info_from_first_row(char *in_line, gal_data_t **datall, int format,
     maxcnum = maxcnum>col->status ? maxcnum : col->status;
 
   /* Go over the line check/fill the column information. */
-  while(++n)
+  while(++ncol)
     {
       /* When we are dealing with a text table, check if there is
          information for this column. For a text image, only the number of
          tokens is important (as the second dimension of the image), so
          just assume there no information. */
       if(format==TXT_FORMAT_TABLE)
-        for(col=*datall; col!=NULL; col=col->next) {if(col->status==n) break;}
+        for(col=*datall; col!=NULL; col=col->next)
+          { if(col->status==ncol) break; }
       else
         col=NULL;
 
@@ -390,15 +393,14 @@ txt_info_from_first_row(char *in_line, gal_data_t **datall, int format,
             }
           else
             {
-              token=strtok_r(n==1?line:NULL, GAL_TXT_DELIMITERS, &line);
+              token=strtok_r(ncol==1?line:NULL, GAL_TXT_DELIMITERS, &line);
               if(token==NULL) break;
-              /* printf(" col %zu: =%s=\n", i, token); */
             }
         }
       else
         {
           /* Make sure a token exists in this undefined column. */
-          token=strtok_r(n==1?line:NULL, GAL_TXT_DELIMITERS, &line);
+          token=strtok_r(ncol==1?line:NULL, GAL_TXT_DELIMITERS, &line);
           if(token==NULL) break;
 
           /* A token exists. For a table, define a new element in the
@@ -410,9 +412,23 @@ txt_info_from_first_row(char *in_line, gal_data_t **datall, int format,
              information). */
           if( *datall==NULL || format==TXT_FORMAT_TABLE )
             {
+              /* Make sure the token is actually a number (or RA/Dec
+                 written in Sexagesimal format) and print a good error
+                 message when the input isn't actually a number but a
+                 string (this test was added because of uncommented
+                 metadata)! */
+              if( gal_type_from_string( &tmpdptr, token, GAL_TYPE_FLOAT64)
+                  && isnan( gal_units_ra_to_degree(token) )
+                  && isnan( gal_units_dec_to_degree(token) ) )
+                error(EXIT_FAILURE, 0, "'%s' couldn't be read as a number "
+                      "(element %zu of first uncommented line) %f ", token,
+                      ncol, gal_units_ra_to_degree(token));
+
+              /* Allocate this column's dataset and set it's 'status' to
+                 the column number that it corresponds to. */
               gal_list_data_add_alloc(datall, NULL, GAL_TYPE_FLOAT64, 0,
                                       NULL, NULL, 0, -1, 1, NULL, NULL, NULL);
-              (*datall)->status=n;
+              (*datall)->status=ncol;
             }
         }
     }
@@ -428,20 +444,21 @@ txt_info_from_first_row(char *in_line, gal_data_t **datall, int format,
 
      Note that 'n' counts from 1, so the total number of tokens is one less
      than 'n'.*/
-  numtokens=n-1;
-  if(format==TXT_FORMAT_IMAGE) n=1;
+  numtokens=ncol-1;
+  if(format==TXT_FORMAT_IMAGE) ncol=1;
 
   /* If the number of columns/images given by the comments is larger than
      the actual number of lines, remove those that have larger numbers from
      the linked list before things get complicated outside of this
      function. */
-  if(maxcnum>n)
+  if(maxcnum>numtokens)
     {
       prev=NULL;
       col=*datall;
       while(col!=NULL)
         {
-          if(col->status > n) /* Column has no data (was only in comments) */
+          /* This column has no data (was only in comments) */
+          if(col->status > numtokens)
             {
               /* This column has to be removed/freed. But we have to make
                  some corrections before freeing it:
@@ -461,7 +478,7 @@ txt_info_from_first_row(char *in_line, gal_data_t **datall, int format,
               gal_data_free(col);
               col=tmp;
             }
-          else                /* Column has data.                          */
+          else /* Column has data. */
             {
               prev=col;
               col=col->next;
@@ -521,7 +538,7 @@ txt_infoll_to_array(gal_data_t *datall, size_t *numdata)
           /* Put all the information from 'data' into the respective part
              of the array. About the pointers, instead of having to
              allocate them again, we will just set them to NULL so
-             'gal_data_free' doesn't remove them.*/
+             'gal_data_free' doesn't remove them. */
           dataarr[ind].flag       = data->flag;    data->flag=0;
           dataarr[ind].name       = data->name;    data->name=NULL;
           dataarr[ind].unit       = data->unit;    data->unit=NULL;
@@ -560,12 +577,12 @@ txt_get_info_line(char *line, gal_data_t **datall, char *comm_start,
 
   switch( gal_txt_line_stat(line) )
     {
-      /* Line is a comment, see if it has formatted information. */
+    /* Line is a comment, see if it has formatted information. */
     case GAL_TXT_LINESTAT_COMMENT:
       txt_info_from_comment(line, datall, comm_start, inplace);
       break;
 
-      /* Line is actual data, use it to fill in the gaps.  */
+    /* Line is actual data, use it to fill in the gaps.  */
     case GAL_TXT_LINESTAT_DATAROW:
       ++dsize[0];
       if(*firstlinedone==0)
@@ -644,8 +661,8 @@ txt_get_info(char *filename, gal_list_str_t *lines, int format,
          lines, but also confirm/complete the info by parsing the first
          uncommented line. */
       while( getline(&line, &linelen, fp) != -1 )
-        txt_get_info_line(line, &datall, comm_start, &firstlinedone, format,
-                          dsize, 1);
+        txt_get_info_line(line, &datall, comm_start, &firstlinedone,
+                          format, dsize, 1);
 
 
       /* Clean up and close the file. */
@@ -735,7 +752,7 @@ txt_read_token(gal_data_t *data, gal_data_t *info, char *token,
   double      *d = data->array,    *db;
 
   /* See if this token is blank. */
-  int isblankstr = ( info->flag==GAL_TABLEINTERN_FLAG_ARRAY_IS_BLANK_STRING
+  int isblankstr = ( info->flag & GAL_TABLEINTERN_FLAG_ARRAY_IS_BLANK_STRING
                      ? ( strcmp(info->array,token)==0 ? 1 : 0 )
                      : 0);
 
@@ -1087,6 +1104,7 @@ txt_read(char *filename, gal_list_str_t *lines, size_t *dsize,
               out->dsize=out->array=NULL;
             }
         }
+      gal_list_data_reverse(&out);
       break;
 
 
@@ -1206,12 +1224,16 @@ gal_txt_image_read(char *filename, gal_list_str_t *lines, size_t minmapsize,
 static int
 txt_stdin_has_contents(long timeout_microsec)
 {
+  int sout;
   fd_set fds;
   struct timeval tv;
 
-  /* Set the timeout time. */
-  tv.tv_sec  = 0;
-  tv.tv_usec = timeout_microsec;
+  /* Set the timeout time. We need to put the number of seconds in 'tv_sec'
+     and the remaining microseconds in 'tv_usec' (this cannot be larger
+     than one million, otherwise 'select' is going to abort with an
+     error). */
+  tv.tv_sec  = timeout_microsec/1000000;
+  tv.tv_usec = timeout_microsec%1000000;
 
   /* Initialize 'fd_set'. */
   FD_ZERO(&fds);
@@ -1219,14 +1241,29 @@ txt_stdin_has_contents(long timeout_microsec)
   /* Set standard input (STDIN_FILENO is 0) as the FD that must be read. */
   FD_SET(STDIN_FILENO, &fds);
 
-  /* 'select' takes the last file descriptor value + 1 in the fdset to
-     check, the fdset for reads, writes, and errors.  We are only passing
-     in reads.  the last parameter is the timeout.  select will return if
-     an FD is ready or the timeout has occurred. */
-  select(STDIN_FILENO+1, &fds, NULL, NULL, &tv);
+  /* From Glibc: "The 'select' function blocks the calling process until
+     there is activity on any of the specified sets of file descriptors, or
+     until the timeout period has expired". 'select' takes the last file
+     descriptor value+1 in the 'FD_SET' above as first argument. If the
+     second (reading), third (writing) and fourth (exception) are not NULL,
+     then it will check for the respective property(s).
 
-  // return 0 if STDIN is not ready to be read.
-  return FD_ISSET(STDIN_FILENO, &fds);
+     When successful (file descriptor has input for the desired action),
+     'select' will return 1. When the timeout has been reached, it will
+     return 0 and when there was an error it will return -1. If there is an
+     error, we'll abort the program and ask the user to contact us (its a
+     bug).*/
+  errno=0;
+  sout=select(STDIN_FILENO+1, &fds, NULL, NULL, &tv);
+  if(sout==-1)
+    error(EXIT_FAILURE, errno, "%s: a bug! Please contact us at '%s' "
+          "to fix the problem. The 'select' function has detected an "
+          "error", __func__, PACKAGE_BUGREPORT);
+
+  /* By this point, 'sout' only has a value of 1 (stdin is ready for
+     reading) or 0 (timeout was reached with no change, so it should't be
+     used). So simply return the value of 'sout'. */
+  return sout;
 }
 
 
@@ -1240,7 +1277,7 @@ gal_txt_stdin_read(long timeout_microsec)
   gal_list_str_t *out=NULL;
   size_t lineno=0, linelen=10;/* 'linelen' will be increased by 'getline'. */
 
-  /* If there is nothing  */
+  /* Only continue if standard input has any contents. */
   if( txt_stdin_has_contents(timeout_microsec) )
     {
       /* Allocate the space necessary to keep a copy of each line as we
@@ -1513,6 +1550,11 @@ txt_write_keys(FILE *fp, struct gal_fits_list_key_t **keylist)
                   tmp->title);
           if(tmp->tfree) free(tmp->title);
         }
+      else if (tmp->fullcomment)
+        {
+          fprintf(fp, "# %s\n", tmp->fullcomment);
+          if(tmp->fcfree) free(tmp->fullcomment);
+        }
       else
         {
           /* For a string type, we need to return a pointer to the
