@@ -6,7 +6,7 @@
 # Original author:
 #     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
-# Copyright (C) 2015-2021, Free Software Foundation, Inc.
+# Copyright (C) 2015-2022 Free Software Foundation, Inc.
 #
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
@@ -59,5 +59,4 @@ if [ ! -f $cat      ]; then echo "$cat does not exist.";   exit 77; fi
 # string. Such programs will execute the command if present and help in
 # debugging when the developer doesn't have access to the user's system.
 $check_with_program $execname $cat --background=$img --mforflatpix \
-                              --clearcanvas --type=int32           \
-                              --output="clearcanvas.fits"
+                    --clearcanvas --type=int32 --output="clearcanvas.fits"
